*-- Scatterplot matrix of Mosaic displays for pairwise association;
/*
   Name:   mosmat.sas
   Title:  Scatterplot matrix of mosaic displays

	Input parameters are the same as for mosaic, except that vnames
	may contain two rows -- 
	vnames[1,] -- long names (split on '/'), used to label diagonal
		panels
	vnames[2,] -- short variable names used in pairwise mosaics

   Set 
	fittype='PARTIAL'; plots=3;	for partial/conditional association
	fittype=anthing;   plots=2		for marginal association
*/
	
start mosmat(dim, table, vnames, lnames, plots, title)
      global(config, devtype, fittype, filltype, shade, space, split,
             htext, verbose, font);
   factors = max(nrow(dim), ncol(dim));
	if ncol(vnames) = 1 then vnames = vnames`;
   if nrow(vnames) = 2 & ncol(vnames)=factors then
      do;
         vnlong = vnames[1,];    *-- long names (diag panels);
         vnames = vnames[2,];    *-- short names (mosaics);		
      end;
      else do;
         vnlong = vnames;
         vnames = vnames;
      end;
	if type(htext) ^= 'N' then htext=factors;
	if type(split) ^= 'C' then split={V H};

	*-- Set default font based on device driver name;
   if type(font ) ^= 'C' then do;
		call execute('device  = upcase("&sysdevic");');
		if index(device,'PS') > 0 then
         font= 'hwpsl009';         /* Helvetica for PS drivers */
		else /* if device='WIN' then */
			font = 'SWISS';
		end;

	
   call gstart;
   ip = 0;  ig=0;      *-- panel and graph numbers;
   replay = '';        *-- greplay list;
   do row = 1 to factors;
      do col = 1 to factors;
         ip = ip+1;
         ig = ig+1;
         if row = col then do;
            run vpanel(vnlong[col]);
            end;
         else do;
            others = remove( (1:factors), (row||col) );
            * transpose table to conform to model;
            order = col||row||others;
            dm = dim;
            vn = vnames;
            ln = lnames;
            tab = table;
				if type(config)='N' then do;
					run modname(config,vn,model);
					print row col model config;
				end;
            run transpos(dm, tab, vn, ln, order);
            ord = rowcatc(vn`);
            titl = title;
            run mosaic(dm, tab, vn, ln, plots, titl);
            end;
         *-- Construct replay list: graph ig in panel ip;
         replay = replay + trim(char(ip,2))+':'+trim(left(char(ig)))+' ';
         end; /* do row */
      end; /* do col */
 
      nvar = '%let nvar =' + char(factors,2) + ';' ;
      replay = '%let replay =' + replay + ';' ;
      call execute(replay);
      call execute(nvar);
 
finish;

*-- Draw a panel for a variable name; 
start vpanel(name)
	global(font, htext);
   *-- Draw a panel with the variable name;
   call gstart;
   call gopen;
   window = {0 0 100 100};
   call gwindow(window);
   call ggrid( {5  95}, {5  95});
   run split(name, '/', lines);
 	nlines = nrow(lines);

   ht=3#htext ;
	*-- Find length of all lines, reduce ht if necessary;
   call gstrlen(len,lines, ht, font);
*	print  lines len;
	if max(len) > 90 
		then ht = round(ht # 90/max(len),.1);
	
   do l = 1 to nlines;
      line = lines[l,];
      call gstrlen(len,line, ht, font);
      x = 50 - len/2;
      y = 50 - ht/2 + 1.5#(ht+1)#(nlines-l);
      call gscript(x,y, line,,,ht,font);
	*	print 'vpanel:' l x y line len ht;
      end;
finish;

start split(in, char, out);
   *-- split a string into separate strings at each occurrence of 'char';
   free out;
   i=1;
   sub = scan(in,i,char);
   do while(sub ^=' ');
      out = out // sub;
      i = i+1;
      sub = scan(in,i,char);
   end;
finish;

